from vbyscan import cores


def get_version(req, target, info_cb, found_cb, not_found_cb):
    def parse_version(data):
        regex = r"([vV][bB]ulletin [\d.]+ [\w\d -_]+)"
        return cores.parse_regex(data, regex)

    def parse_version_from_content(data):
        regex = r"meta name=\"generator\" content=\"(.*?)\" />"
        return cores.parse_regex(data, regex)

    # def parse_version_from_link(data):
    #     # https://github.com/OWASP/vbscan/blob/2b1ce48c11b2d73127da25c6e2bb66edf8236b2d/core/ver.pl#L47
    #     regex = r"vbulletinlink\">vBulletin\&reg\;\<\/a\> (.*?)br"
    #     return cores.parse_regex(data, regex)
    #
    # def parse_version_from_simple_version(data):
    #     # https://github.com/OWASP/vbscan/blob/2b1ce48c11b2d73127da25c6e2bb66edf8236b2d/core/ver.pl#L57
    #     regex = r"SIMPLEVERSION = \"(.*?)\""
    #     result = cores.parse_regex(data, regex)

    def parse_version_from_footer(data):
        regex = r"Powered by(.*?)<br"
        result = cores.parse_regex(data, regex)
        if result:
            return "vBulletin " + result.split("Version")[-1].replace(" ", "")
        else:
            print(data)

    name = "Enumerate Vbulletin version"
    is_found = False
    list_files = [
        "clientscript/vbulletin_menu.js",
        "clientscript/vbulletin_global.js",
        "clientscript/vbulletin-core.js",
    ]

    info_cb(name)

    for file_name in list_files:
        uri = f"{target}{file_name}"
        r = req.get(uri, allow_redirects=True)
        if r.status_code == 200:
            # Code gets version from banner only. We try cut the data to very short text
            cmp_data = "\n".join(r.text.split("\n")[0:5])
            version = parse_version(cmp_data)

            if version:
                found_cb(version)
                return version.split(" ")[1]

    r = req.get(target, allow_redirects=True)
    version = parse_version_from_content("\n".join(r.text.split("\n")[0:10]))
    if version:
        found_cb(version)
        return version.split(" ")[1]

    version = parse_version_from_footer(r.text)
    if version:
        found_cb(version)
        return version.split(" ")[1]

    if not is_found:
        not_found_cb(name)
        return ""
